!=======================================================================
      subroutine odesol(nt,nfield,order,level,itest,nwksp,
     &                  at,bt,Y,K,ierr)
!=======================================================================
      implicit none
      integer nt,nfield,order,level,itest,nwksp,ierr
      real*8 at,bt
      real*8 Y(nfield,0:nt),K(*)

      integer i,j
      real*8 true1,true2,distance
      real*8 freq,pi,omega
      common /c2test/ freq,pi,omega
      real*8 F0,mass,mu
      common /c4test/ F0,mass,mu


!--- check if "order" is okay

      if(order.eq.1 .or. order.eq.2 .or. order.eq.4)then
      else
         print'("ODESOL: ""order"" should be 1, 2, or 4.")'
         print*,"currently, order=",order
         ierr=1
         return
      endif


!--- Print out

      if(level.ge.1) print'("ODESOL: order=",i1)',order
      if(level.ge.2) then
          print*,"at=",at," bt=",bt," nt=",nt," nfield=",nfield
      endif


!--- Set parameters

      freq=2.d0
      pi=dacos(-1.d0)
      omega=2.d0*pi*freq

      F0=40.d0
      mass=1.d0
      mu=2.d0*pi

!-----------------------------------------------
!--- Read/Set the initial values
!-----------------------------------------------

      if(itest.eq.1)then
         if(level.ge.1)then
            print'(1x,33("-"))'
            print'(1x,"itest=1: RUN A GIVEN TEST PROBLEM")'
            print'(1x,"SET itest=0 TO SOLVE YOUR PROBLEM")'
            print'(1x,33("-"))'
         endif
         if(nfield.eq.1)then
            Y(1,0)=0.d0
         else if(nfield.eq.2)then
*            Y(1,0)=1.d0
            Y(1,0)=1.d0+F0/(mass*(omega**2-mu**2))
            Y(2,0)=0.d0
         else
            stop
         endif
      else
         open(68,file="initial.dat")
         do i=1,nfield
            read(68,*) Y(i,0)
         enddo
         close(68)
      endif


!-----------------------------------------------
!--- Now, call the solvers depending "order"
!-----------------------------------------------

      if(order.eq.1) then

         call euler(nt,nfield,at,bt,Y,K,level,ierr)

      else if(order.eq.2) then

         call rk2(nt,nfield,at,bt,Y,K,level,ierr)

      else if(order.eq.4) then

         call rk4(nt,nfield,at,bt,Y,K,level,ierr)

      endif


!-----------------------------------------------
!--- Numerical Analysis
!-----------------------------------------------

      if(level.le.0) return

      print'(40("-"))'
      print*,"order=",order," nfield=",nfield," itest=",itest
      print*,"at=",at," bt=",bt,"  nt=",nt," ht=",(bt-at)/dble(nt)
      print'(" Y(.,nt)=",$)'
      print*,(Y(i,nt),i=1,nfield)

      if(itest.eq.1)then
         if(nfield.eq.1)then
            true1=dexp(bt)-bt-1.d0
            print*,"True   =",true1
            distance=dabs(true1-Y(1,nt))
         else if(nfield.eq.2)then
*            true1=dcos(omega*bt)
*            true2=dsin(omega*bt)
            true1=dcos(omega*bt)+F0*dcos(mu*bt)/(mass*(omega**2-mu**2))
            true2=dsin(omega*bt)
     &           +F0*mu*dsin(mu*bt)/(omega*mass*(omega**2-mu**2))
            print*,"True   =",true1,true2
            distance=dsqrt((true1-Y(1,nt))**2+(true2-Y(2,nt))**2)
         endif
         print'(" Error  =",1pe9.2)',distance
      endif

      print'(40("-"))'

      if(level.ge.2.and.nfield.eq.2)then
         open(1,file="fort.1")
         write(1,*) nt,bt
         close(1)
         open(68,file="fort.sol2")
         do j=0,nt
         write(68,*) (Y(i,j),i=1,2)
         enddo
         close(68)
      endif

      return
      end

